/*
PeRdr - PE file disassembler
Copyright (C) 1999-2003 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include <cctype>
#include <algorithm>
#include "codeglob.h"

using namespace std;

// Data analisy code
// conta i caratteri della stringa e i caratteri stampabili
// ritorna 0 in tutti e due i risultati se errore di lettura
// !!! aggiungere stringhe Unicode
void CodeParser::GetStringStats(vma_t address,unsigned& len,unsigned& num_printable)
{
  try
  {
		ObjectModule::DataReader reader = module->GetDataReader(address);
    unsigned n = 0;
    num_printable = 0;
    for (;;)
    {
      // if relocation return (no relocation on strings)
      if (module->HasRelocation()
          && module->GetRelocationInfos().GetRelocation(reader.Tell()))
      {
        len = num_printable = 0;
        return;
      }

			// !!! if program try to get statistics for already setted strings this fail
      // if code busy return
      if (byteInfo.IsOccupied(reader.Tell()))
      {
        len = num_printable = 0;
        return;
      }

      uint8_t car = reader.ReadU8();
      if (isprint(car))
        ++num_printable;
      ++n;
      if (car == 0)
        break;
    }
    len = n;
  }
  catch(const OutOfAddress&)
  {
    len = num_printable = 0;
  }
}

void CodeParser::GetPointerArrayStats(vma_t address,PointerArrayStat& stat)
{
  // inizializza statistiche
  stat.nCodePointer = 0;
  stat.nPointer     = 0;
  stat.nNull        = 0;
  stat.nFirstNull   = 0;
  bool bFirstNull = false;
  try
  {
		ObjectModule::DataReader reader = module->GetDataReader(address);
    for (unsigned n=0;;++n)
    {
      // testa sovrapposizione
      if (byteInfo.IsOccupied(reader.Tell(),addr_bytes))
        break;
      // !!!
      vma_t addr = reader.ReadU32LE();
      // se puntatore nullo incrementa e testa se primo
      if (IsNullAddress(addr))
      {
        ++stat.nNull;
        // se primo puntatore nullo segna posizione
        if (!bFirstNull)
          stat.nFirstNull = n;
        bFirstNull = true;
      }
      // fermati primo indirizzo invalido non nullo
      else if (!module->IsValid(addr))
        break;
      ++stat.nPointer;
      if (module->GetSection(addr)->IsCode())
        ++stat.nCodePointer;
    }
  }
  catch(const OutOfAddress&) {}; // se errore lettura si ferma
  // se non trovati puntatori nulli metti il primo puntatore nullo cella dopo
  if (!bFirstNull)
    stat.nFirstNull = stat.nPointer;
}

void CodeParser::GetBitStat(vma_t address,unsigned numBytes,BitStat& stat)
{
  // init counters
  fill(stat.bit8,stat.bit8+8,0);  
  fill(stat.bit16,stat.bit16+16,0);  
  fill(stat.bit32,stat.bit32+32,0);
  
  unsigned n;
  try
  {
		ObjectModule::DataReader reader = module->GetDataReader(address);
    for (n=0;n<numBytes;++n)
    {
      if (byteInfo.IsOccupied(reader.Tell()))
        break;
      uint8_t byte = reader.ReadU8();

      // increment counters
      unsigned *pCount = &stat.bit32[(n%4u)*8u];
      for(unsigned n=1;n != (1<<8);++pCount,n<<=1)
        if(byte & n)
          ++*pCount;
    }
  }
  catch(const OutOfAddress&) {}; // se errore lettura si ferma

  // group counters
  for(n=0;n < 16;++n)
    stat.bit16[n] = stat.bit32[n] + stat.bit32[n+16];
  for(n=0;n < 8;++n)
    stat.bit8[n] = stat.bit16[n] + stat.bit16[n+8];
}
