/*
PeRdr - PE file disassembler
Copyright (C) 1999-2002 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "param.h"

unsigned Param::memTypeSizes[Param::memTypeCount] =
{
#define DEFINE_MEMTYPE(name,size) size,
#define DEFINE_MEMTYPEI(name,size) size,
#include "memtype.h"
#undef DEFINE_MEMTYPE
#undef DEFINE_MEMTYPEI
};

extern void ReadInstruction(const char* s);
extern void ReadParam(Param& prm,const char* s);

Param::Param(enum TCRegistry ,reg_t reg):
	relocation(NULL)
{
  type = t_registry;
  mem_reg1 = reg;
  mem_reg2 = null_reg;
  sm.size = 0;
  // size = ?? from reg
}

Param::Param(enum TCLiteral ,int _literal,unsigned char _size):
	relocation(NULL)
{
  type = t_literal;
  mem_reg1 = null_reg;
  mem_reg2 = null_reg;
  literal = _literal;
  sm.size = _size;
}

Param::Param(enum TCFarLiteral ,int _literal,unsigned char _size,int segment):
	relocation(NULL)
{
  type = t_farliteral;
  mem_reg1 = static_cast<reg_t>(segment); // !!!
  mem_reg2 = null_reg;
  literal = _literal;
  sm.size = _size;
}

Param::Param(enum TCMemory ,int address,MemoryTypes _memType):
	relocation(NULL)
{
  type = t_memory;
  mem_reg1 = null_reg;
  mem_reg2 = null_reg;
  literal = address;
  sm.memType = _memType;
}

Param::Param(enum TCMemory ,reg_t reg,int address,MemoryTypes _memType):
	relocation(NULL)
{
  type = t_memory;
  mem_reg1 = reg;
  mem_reg2 = null_reg;
  literal = address;
  sm.memType = _memType;
}

Param::Param(enum TCMemory ,reg_t reg1,reg_t reg2,int address,unsigned char multiplier,MemoryTypes _memType):
	relocation(NULL)
{
  type = t_memory;
  mem_reg1 = reg1;
  mem_reg2 = reg2;
  literal = address;
  factor = multiplier;
  sm.memType = _memType;
}

#ifdef DEBUG
void Param::InitInvalid()
{
  sm.size = 0;
}

void Param::CheckValid()
{
  if (type == t_literal)
  {
    _PRG_ASSERT(sm.size != 0);
  }
}
#endif

unsigned char Param::GetSize() const
{
  switch (type)
  {
  default:
    _PRG_ASSERT(0);
  case t_memory:
    return GetMemSize();
  case t_literal:
  case t_farliteral:
    return GetLiteralSize();
  case t_registry:
    return RegistryInfo::GetInfo(mem_reg1)->GetByteSize();
  }
}

