/*
PeRdr - PE file disassembler
Copyright (C) 1999-2003 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include <getopt.h>
#include "perdroptions.h"

PeRdrOption options = {
  false, // showBytes
  false, // disableP3
  false, // disable3DNow
  false, // disableEnh3DNow
  false, // disableAthlon
  false, // showVersion
  false, // showHelp
  false, // showPriority
  false, // rawDumping
  false, // extractIconRes
  false, // showHeaders
  false, // showImports
  false, // showRelocations
  false, // showExports
  false, // showResources
  false  // showCode
};

static struct option long_options[] = {
  {"help", 0, 0, 'h'},
  {"version", 0, 0, 'v'},
  {"bytes", 0, 0, 'b'},
  {"show-priority", 0, 0, 1},
  {"raw-dump", 0, 0, 2},
  {"extract-icon-resource", 0, 0, 3},
  {"disable-3dnow", 0, 0, 4},
  {"disable-enh3dnow", 0, 0, 5},
  {"disable-pentium3", 0, 0, 6},
  {"disable-athlon", 0, 0, 7},
  {"show-headers", 0, 0, 8},
  {"show-imports", 0, 0, 9},
  {"show-relocations", 0, 0, 10},
  {"show-exports", 0, 0, 11},
  {"show-resources", 0, 0, 12},
  {"show-code", 0, 0, 13},
  {"addr-start", 1, 0, 14 },
  {"addr-end", 1, 0, 15 },
  {0, 0, 0, 0}
};

int ReadOptions(int argc,char** argv)
{
  int c, option_index, res;

  res = argc - 1; 
  while ((c = getopt_long (argc, argv, "b", long_options, &option_index)) != -1) {
    switch (c) {
      case 'h': options.showHelp = true; break;
      case 'v': options.showVersion = true; break;
      case 'b': options.showBytes = true; break;
      case 1: options.showPriority = true; break;
      case 2: options.rawDumping = true; break;
      case 3: options.extractIconRes = true; break;
      case 4: options.disable3DNow = options.disableEnh3DNow = true; break;
      case 5: options.disableEnh3DNow = true; break;
      case 6: options.disableP3 = true; break;
      case 7: options.disableAthlon = true; break;
      case 8: options.showHeaders = true; break;
      case 9: options.showImports = true; break;
      case 10: options.showRelocations = true; break;
      case 11: options.showExports = true; break;
      case 12: options.showResources = true; break;
      case 13: options.showCode = true; break;
      case 14: options.addrStart = strtoul(optarg, NULL, 0); break;
      case 15: options.addrEnd = strtoul(optarg, NULL, 0); break;
      default: res = optind;
    }
  }

  if (!options.showHeaders && !options.showImports && !options.showRelocations 
		  && !options.showExports && !options.showCode
		  && !options.showResources)
  {
    options.showHeaders = true;
    options.showImports = true;
    options.showRelocations = true;
    options.showExports = true;
    options.showCode = true;
    options.showResources = true;
  }
	    
  return res;
}
