/*
PeRdr - PE file disassembler
Copyright (C) 1999-2002 Frediano Ziglio
-----

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-----

INFORMATION
  www: http://freddy77.tripod.com/perdr/
  e-mail: freddy77@angelfire.com
*/
#include "global.h"
#ifdef HAVE_HDRSTOP
#pragma hdrstop
#endif

#include "persrc.h"
#include <stdexcept>
#include <list>
#include "utils/f77auto_ptr"

struct iconinfo
{
  // id of icon must be numeric
  unsigned id;
  long pos;
  uint32_t size;
};

typedef std::list<iconinfo> TIconInfos;

static void CollectIconInfos(ICFile& file,const PeResourceDirectoryData* data,const PeResourceDirectoryInfo* info,unsigned level,void* param)
{
  _DEBUG_(fprintf(stderr,"data %p level %d name %p id %d\n", data, level, info->name, info->id ));
  if (data != NULL && level == 3 && info->name == NULL && info->highLevel->id == 3)
  {
    _DEBUG_(fprintf(stderr,"---data %p level %d name %p id %d\n", data, level, info->name, info->highLevel->id));
    iconinfo iinfo;
    iinfo.id = info->id;
    iinfo.pos = file.Tell();
    iinfo.size = data->data.Size;
    _DEBUG_(fprintf(stderr,"---id %d pos %ld size %d\n", iinfo.id, iinfo.pos, iinfo.size));
    
    (*reinterpret_cast<TIconInfos*>(param)).push_back(iinfo);
  }
}

struct WriteIconParam
{
  TIconInfos iconInfos;
  int n;
  WriteIconParam():n(1) {};

};
  
static const iconinfo* findIconInfo(const TIconInfos& infos,unsigned id)
{
  TIconInfos::const_iterator end = infos.end();
  for(TIconInfos::const_iterator i = infos.begin(); i != end; ++i)
    if ( (*i).id == id )
      return &(*i);
  return NULL;
}

static void CopyFileToFile(const ICFile& from,ICFile& to,size_t bytes)
{
  char buffer[128];
  while(bytes > 0)
  {
    size_t len = f77_min(bytes,(size_t)128);
    from.RawRead(buffer,len);
    to.RawWrite(buffer,len);
    bytes -= len;
  }
}

static void WriteIcons(ICFile& file,const PeResourceDirectoryData* data,const PeResourceDirectoryInfo* info,unsigned level,void* param)
{
  static const char zeroes[2] = { 0, 0 };
  if (data != NULL && level == 3 && info->highLevel->id == 14)
  {
    WriteIconParam *p = reinterpret_cast<WriteIconParam*>(param);
    
    // ok, write this icon
    NEWHEADER hdr;
    RawRead(file,hdr);
    
    char buffer[128];
    sprintf(buffer,"resico%d.ico",p->n);
    ++p->n;
    CFile ico(buffer,"wb");
    
    // save header
    RawWrite(ico,hdr);
    
    
    freddy77::auto_delete_ptr<const iconinfo*> icons(new const iconinfo*[hdr.ResCount]);
    int i;
    for( i = 0; i < hdr.ResCount; ++i )
    {
      RESDIR resico;
      // !!! fix pack problem
      file.RawRead(&resico,14);
      
      // save icon header
      // !!! fix pack problem
      ico.RawWrite(&resico,14);
      ico.RawWrite(zeroes,2);
      
      // find icon and save it
      const iconinfo* info = findIconInfo(p->iconInfos,resico.IconCursorId);
      *(icons.get()+i) = info;
      if (!info)
        throw std::runtime_error("icon not found for icon group");
      if (resico.BytesInRes != info->size)
        throw std::runtime_error("icon with wrong size");
      // !!! check icon dimension and colors
    }
    for( i = 0; i < hdr.ResCount; ++i )
    {
      const iconinfo* info = *(icons.get()+i);
      file.Seek(info->pos);
      CopyFileToFile(file,ico,info->size);
    }
  }
}

void ResourceExtract::ExtractIcons(PeFile& peFile)
{
//  fprintf(stderr,"sizeof hdr %d\n",sizeof(RESDIR));
//  throw std::runtime_error("ferma");
  // parse first for information retrieval (position and size of resources)
  WriteIconParam param;
  peFile.ParseResource(CollectIconInfos,&param.iconInfos);

  // parse for output files
  peFile.ParseResource(WriteIcons,&param);
}
